package Device::VCR::SSP::Serial;
use strict;
use warnings;
require Exporter;

BEGIN {
    use vars qw( $OS_win  %R_CODES);
    %R_CODES = (
        R_ACK        => 0x0a,
        R_NAK        => 0x0b,
        R_ERROR      => 0x02,
        R_COMPLETION => 0x01,
        R_NOT_TARGET => 0x05,
    );
}

#use vars qw( $OS_win  %R_CODES);

our @ISA = qw(Exporter);

our %EXPORT_TAGS = (
    'all'          => [ %R_CODES, keys %R_CODES ],
    'return_codes' => [ keys %R_CODES ]
);

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(

);

our $VERSION = '0.01';

BEGIN {
    use constant \%R_CODES;

    no strict 'refs';

    # Bits returned by STATUS_SENSE
    my %BT_SS = (
        SS_ERROR          => 0,
        SS_CASSETTE_OUT   => 3,
        SS_REC_INHIBIT    => 4,
        SS_WARNING        => 11,
        SS_REPEAT_MODE    => 18,
        SS_ON_SCREEN_ON   => 20,
        SS_COUNTER_MEMORY => 21,
        SS_TIMER_REC_ON   => 22,
        SS_REC_MODE       => 25,
        SS_EJECT          => 26,
        SS_RVS_PLAY       => 27,
        SS_STOP_MODE      => 28,
        SS_REW_MODE       => 29,
        SS_FFWD_MODE      => 30,
        SS_PLAY_MODE      => 31,
        SS_STILL_MODE     => 32,
        SS_PAUSE_MODE     => 33,
        SS_REVIEW_MODE    => 36,
        SS_CUE_MODE       => 37,
    );

    # Make bit accessors methods;
    foreach ( keys %BT_SS ) {
        *{$_} = sub {
            my $self = shift;
            substr( $self->{BT_SS}, $BT_SS{$_}, 1 );
        };
    }

    sub dump_status_sense {
        my $self = shift;
        my $text = "Dumping Status...\n";
        $text .= $self->{BT_SS} . "\n";
        foreach ( sort { $BT_SS{$a} <=> $BT_SS{$b} } keys %BT_SS ) {
            $text .=
              "[$BT_SS{$_}] $_  = "
              . substr( $self->{BT_SS}, $BT_SS{$_}, 1 ) . "\n";
        }
        $text;
    }

    # Bits returned by TL_STATUS_SENSE
    my %BT_TLSS = (
        TLSS_AUDIO_ON          => 24,
        TLSS_POWER_ON          => 25,
        TLSS_TIMER_REC_ON      => 28,
        TLSS_SERIES_REC_ON     => 29,
        TLSS_ONESHOT_REC_ON    => 30,
        TLSS_ALARM_REC_ON      => 31,
        TLSS_ALARM_SEARCH_SET  => 33,
        TLSS_ALARM_SEARCH_MODE => 34,
        TLSS_ALARM_SCAN_MODE   => 36,
        TLSS_TD_SEARCH_SET     => 37,
        TLSS_TD_SEARCH_MODE    => 38,
        TLSS_MENU_MODE         => 39,

    );

    # Make bit accessors methods
    foreach ( keys %BT_TLSS ) {
        *{$_} = sub {
            my $self = shift;
            vec( $self->{BT_TLSS}, $BT_TLSS{$_}, 1 );
        };
    }

    sub dump_tl_status_sense {
        my $self = shift;
        my $text = "Dumping TL Status...\n";
        $text .= $self->{BT_TLSS} . "\n";
        foreach ( sort { $BT_TLSS{$a} <=> $BT_TLSS{$b} } keys %BT_TLSS ) {
            $text .=
              "[$BT_TLSS{$_}] $_  = "
              . substr( $self->{BT_TLSS}, $BT_TLSS{$_}, 1 ) . "\n";
        }
        $text;
    }

}

# Command table:
# NAME=> {
#    cmd=>0x00,
#    rcv=>"bytes to receive",
#    fixed=> If true, doesn't send data after initial request
#    wait=> Float time in seconds to wait after sending command
#    fixup=> sub {} Post process received data, defaults to ord(byte)
# }

my %CMD = (
    'CLEAR' => {
        'cmd'   => 86,
        'desc'  => 'Cancel Error/Clear Commands',
        'fixed' => 1,
        'rcv'   => 1
    },
    'CLEAR_ERROR' => {
        'cmd'   => 65,
        'desc'  => 'Cancel Error/Clear Commands',
        'fixed' => 1,
        'rcv'   => 1
    },
    'CLOCK_ADJUST' => {
        'cmd'   => 224,
        'desc'  => 'Reset Minutes/Second To Closest Hour',
        'fixed' => 1,
        'rcv'   => 1
    },
    'COUNTER_CODE' => {
        'cmd'   => 208,
        'desc'  => 'Retrieve Counter Position',
        'fixed' => 1,
        'fixup' => sub { shift  },
        'rcv'   => 6
    },
    'COUNTER_MEMORY_TOGGLE' => {
        'cmd'   => 227,
        'desc'  => 'Turn On/Off Counter Memory',
        'fixed' => 1,
        'rcv'   => 1
    },
    'COUNTER_RESET' => {
        'cmd'   => 226,
        'desc'  => 'Reset the Counter',
        'fixed' => 1,
        'rcv'   => 1
    },
    'EJECT' => {
        'cmd'   => 163,
        'desc'  => 'Eject',
        'fixed' => 1,
        'rcv'   => 2
    },
    'FWD' => {
        'cmd'   => 171,
        'desc'  => 'FFWD/FWD/ADV/ALARMSCAN',
        'fixed' => 1,
        'rcv'   => 1
    },
    'HEAD_TIME' => {
        'cmd'   => 210,
        'desc'  => 'Headtime Usage in Hours',
        'fixed' => 1,
        'fixup' => sub { shift  },
        'rcv'   => 5
    },
    'MENU' => {
        'cmd'   => 116,
        'desc'  => 'Menu Display/Menu Switch',
        'fixed' => 1,
        'rcv'   => 1
    },
    'MENU_EXIT_AUDIO_ON_SEARCH' => {
        'cmd'   => 34,
        'desc'  => 'Menu Exit/Audio On/SearchModes',
        'fixed' => 1,
        'rcv'   => 1
    },
    'MENU_RESET' => {
        'cmd'   => 225,
        'desc'  => 'Reset Menu to Defaults.',
        'fixed' => 1,
        'rcv'   => 1
    },
    'ON_SCREEN_ON_TOGGLE' => {
        'cmd'   => 78,
        'desc'  => 'Toggle On Screen Display',
        'fixed' => 1,
        'rcv'   => 1
    },
    'PAUSE_STILL' => {
        'cmd'   => 79,
        'desc'  => 'Still/Pause',
        'fixed' => 1,
        'rcv'   => 1
    },
    'PLAY' => {
        'cmd'   => 58,
        'desc'  => 'Play/Record Check',
        'fixed' => 1,
        'rcv'   => 1
    },
    'POWER_TIME' => {
        'cmd'   => 211,
        'desc'  => 'Power Time In Hours',
        'fixed' => 1,
        'fixup' => sub { shift  },
        'rcv'   => 5
    },
    'POWER_TOGGLE' => {
        'cmd'   => 123,
        'desc'  => 'Power On/Off Toggle',
        'fixed' => 1,
        'rcv'   => 1
    },
    'QM_SECURE_OFF' => {
        'cmd'   => 106,
        'desc'  => 'Quick Secure Lock Off',
        'fixed' => 1,
        'rcv'   => 1
    },
    'QM_SECURE_ON' => {
        'cmd'   => 105,
        'desc'  => 'Quick Secure Lock On',
        'fixed' => 1,
        'rcv'   => 1
    },
    'RECORD' => {
        'cmd'   => 202,
        'desc'  => 'Record',
        'fixed' => 1,
        'rcv'   => 1
    },
    'REC_DUB_REQUEST' => {
        'cmd'   => 250,
        'desc'  => 'Request record',
        'fixed' => 1,
        'rcv'   => 1
    },
    'REV_PLAY' => {
        'cmd'   => 74,
        'desc'  => 'Reverse Play',
        'fixed' => 1,
        'rcv'   => 1,
        'wait'  => 4
    },
    'REW' => {
        'cmd'   => 172,
        'desc'  => 'REW/REV/ADV/ALARMSCAN',
        'fixed' => 1,
        'rcv'   => 1
    },
    'SET_RP_DOWN' => {
        'cmd'   => 102,
        'desc'  => 'Change Selection/Decrease Value/Select Tape Speed -',
        'fixed' => 1,
        'rcv'   => 1
    },
    'SET_RP_UP' => {
        'cmd'   => 101,
        'desc'  => 'Change Selection/Increase Value/Select Tape Speed +',
        'fixed' => 1,
        'rcv'   => 1
    },
    'SHARPNESS_SHARP' => {
        'cmd'   => 77,
        'desc'  => 'Sharpen Image',
        'fixed' => 1,
        'rcv'   => 1
    },
    'SHARPNESS_SOFT' => {
        'cmd'   => 76,
        'desc'  => 'Soften Image',
        'fixed' => 1,
        'rcv'   => 1
    },
    'SHIFT_DOWN' => {
        'cmd'   => 100,
        'desc'  => 'Clock Previous/Main Menu/Superimose Down',
        'fixed' => 1,
        'rcv'   => 1
    },
    'SHIFT_RIGHT' => {
        'cmd'   => 99,
        'desc'  => 'Menu Accept/Sub Menu/Superimpose Right',
        'fixed' => 1,
        'rcv'   => 1
    },
    'STATUS_SENSE' => {
        'cmd'   => 215,
        'desc'  => 'Status Sense',
        'fixed' => 1,
        'fixup' => sub { unpack( "b*", shift)  },
        'rcv'   => 5
    },
    'STOP' => {
        'cmd'   => 63,
        'desc'  => 'Stop',
        'fixed' => 1,
        'rcv'   => 1
    },
    'TIMER_TOGGLE' => {
        'cmd'   => 96,
        'desc'  => 'Timer On/Off Toggle',
        'fixed' => 1,
        'rcv'   => 1
    },
    'TL_OFF' => {
        'cmd'   => 247,
        'desc'  => 'TL Command Off',
        'fixed' => 1,
        'rcv'   => 1
    },
    'TL_ON' => {
        'cmd'   => 246,
        'desc'  => 'TL Command On',
        'fixed' => 1,
        'rcv'   => 1
    },
    'TL_STATUS_SENSE' => {
        'cmd'   => 214,
        'desc'  => 'TL Status Sense',
        'fixed' => 1,
        'fixup' => sub { unpack( "bbbb*", shift) },
        'rcv'   => 5
    },
    'TRACKING_CENTER' => {
        'cmd'   => 82,
        'desc'  => 'Set Tracking to Neutral',
        'fixed' => 1,
        'rcv'   => 1
    },
    'TRACKING_VLOCK_SHIFT_DOWN' => {
        'cmd'   => 81,
        'desc'  => 'Tracking Down/Vertical Align Down/Shift Down-Menu Mode',
        'fixed' => 1,
        'rcv'   => 1
    },
    'TRACKING_VLOCK_SHIFT_UP' => {
        'cmd'   => 80,
        'desc'  => 'Tracking Up/Vertical Align Up/Shift Up-Menu Mode',
        'fixed' => 1,
        'rcv'   => 1
    }
);


# Preloaded methods go here.
sub new {
    my $class = shift;
    my %args  = (
        port_obj =>
          undef,    # Expects a Win32::SerialPort or Device::Serialport obj
        debug         => undef,
        _port_nocheck => undef,    # Set to skip port_obj validation
        @_,
    );
    my $self = {@_};
    if ( $self->{debug} ) {
        $self->{_debug} = $self->{debug};
    }

    # TODO Check for valid port reference
    if ( $self->{port_obj}->isa('Device::SerialPort') ) {
        warn "Seiral port is a Device::SerialPort\n" if $args{debug};
    }
    elsif ( $self->{port_obj}->isa('Win32::SerialPort') ) {
        warn "Seiral port is a Win32::SerialPort\n" if $args{debug};
    }
    elsif ( $args{_port_nocheck} ) {
        warn "Don't know what kind of port obj, but allowing." if $args{debug};
    }
    else {
        warn
"Invalid port object!, need a Device::SerialPort or Win32::SerialPort!";
        return;
    }

    bless( $self, $class );
    return $self;

}

sub listcmd {
    my $self = shift;
    return sort keys %CMD;

}

sub sendbyte {
    my ( $self, $byte, $receive ) = @_;
    $receive = 1 unless $receive;
    my $Port = $self->{port_obj};
    unless ( defined $byte ) {
        warn "sendbyte: Nothing to send!" and return;
    }

    unless ( $Port->write( pack( "C", $byte ) ) ) {
        warn "Couldn't write to port!";
        return;
    }
    my ( $count, $res ) = $Port->read($receive);

}

sub sendcmd {
    my $self  = shift;
    my $cname = shift;
    my $Port  = $self->{port_obj};

    if ( $self->{_debug} ) {
        warn "sendcmd: $cname\n";
    }

    unless ( exists $CMD{$cname} ) {
        warn "Invalid command [$cname]!";
        return;
    }
    my %cmd = %{ $CMD{$cname} };
    if ( $self->{_debug} ) {
        warn sprintf( "Sending: [0x%02X] %s\n", $cmd{cmd}, $cmd{desc} );
    }

    unless ( $Port->write( pack( "C", $cmd{cmd} ) ) ) {
        warn "Couldn't write to port!";
        return;
    }

    my ( $count, $res ) = $Port->read( $cmd{rcv} );

    if ( $self->{_debug} ) {
        warn sprintf( "Expected: %d\tReceived: %d\tData: 0x%0X \n",
            $cmd{rcv}, $count, unpack( "C", $res ) );
    }

    if ( $count != $cmd{rcv} ) {
        warn "Wanted $cmd{rcv} bytes, but got $count!\n";
    }
    elsif ( defined $cmd{fixup} ) {
        return $cmd{fixup}->($res);
    }
    else {
        unpack( "C" x $cmd{rcv}, $res );
    }

}

sub debug {
    my ( $self, $debug ) = @_;
    $self->{_debug} = $debug;

}

sub status_sense {
    my $self = shift;
    $self->{BT_SS} = $self->sendcmd('STATUS_SENSE');

}

sub tl_status_sense {
    my $self = shift;
    $self->{BT_TLSS} = $self->sendcmd('TL_STATUS_SENSE');

}

1;
__END__


#################### main pod documentation begin ###################
=head1 NAME

Device::VCR::SSP::Serial - Extension for controlling SSP capable Sanyo VCRs over RS232C

=head1 SYNOPSIS

  use Device::VCR::SSP::Serial;
  my $vcr = Device::VCR::SSP::Serial->new(port_obj=>$PORT);
   ...
  my $response = $vcr->sendcmd('PLAY');

=head1 DESCRIPTION

This module is for the control of Sanyo Security VCRs that support the Sanyo SSP (Security Serial Protocol) protocol via RS-232C. I believe this "standard" has been extended in the past few years. As far as compatability with newer devices, I have no idea. This was developed with a Sany SRT6000 recorder and the documentation info was gleaned from a copy of the VZU-485/232C add-on board manual. It should work with any device that has one of these boards at the least.

Currently, a copy of the VZU-485/232C manual is available in PDF form at.
http://www.leeland.net/lib/leeland/sssp.pdf

This module expects to valid Device::SerialPort or Win32::SerialPort object to be past to the constructor. No modification of the port object is done by this module.
  
The protocol is rather loose. Most commands Send one byte and expect one response byte. Though the "SENSE" commands return 5 bytes to indicate status of the VCR. 

Additionally, some commands can send data along with the request. These are not yet implemented, but will be forthcoming shortly.

A complete description can be obtained by reading the VZU-485/232C manual.

There is an example VCR control (vcr.pl) in the scripts subdirectory.
 
 

=head1 USAGE

use Device::VCR::SSP::Serial qw(:return_codes);

my $vcr = Device::VCR::SSP::Serial->new(
              port_obj=>$Port, # Win32::SerialPort/Device::SerialPort obj.
              debug=>1,        # Verbose to STDERR
          );  

$vcr->sendcmd('TL_ON'); # Enable control

$vcr->status_sense; # Retrieve status and store internally.

unless ($vcr->SS_PLAY_MODE){
    my $res = $vcr->sendcmd('PLAY'); # Play if not already in play mode.
    if ($res == R_ACK){
       print "VCR Playing.\n";
    }elsif ($res == R_NAK){
       warn "VCR not playing."
       # Further quering of the status to determine the problem.
    }else{
       warn "Undefined response: [$res]\n";
    }
}

$vcr->sendcmd('TL_OFF'); # Disable control          

=head2 EXPORT

None by default.

=head3 :return_codes

Will export return code constants 
(R_ACK, R_NAK, R_ERROR, R_COMPLETION, R_NOT_TARGET)

=head1 Return Codes

When transmitting to the VCR, the VCR will respond with a return code. These codes are available as exported constants using the :return_codes export tag.

=head2 R_ACK

Returned by the VCR to indicate it has received the command.

=head2 R_NOT_TARGET

Returned if during a search or scan mode, the tape has been completely rewound or advanced without finding the desired target.

Also returned when the mode is canceled.

=head2 R_COMPLETION

Returned when the target is found after a search or scan operation.

Returned every time a recorded alarm is found during a scan operation.

=head2 R_ERROR

When a command of more than 2 bytes cannot be received past the second byte by the VCR, it will return R_ERROR to indicate the command was not received.

Any other command received sent after an R_ERROR will not be received, instead the VCR status will be returned. To cancel this state, send the CLAR_ERROR or CLEAR command.

=head2 R_NAK

Response when an undefined or non-existant command is sent. Unimplemented features will also return this code.

=head1 METHODS

=head2 sendcmd($COMMAND_NAME)

Used to send commands to the VCR. Sense commands should not be sent via sendcmd, but rather the status_sense and tl_status_sense methods.

=head3 CLEAR

To clear all commands.

To cancel an error state.  

=head3 CLEAR_ERROR
  
To clear the last input (sent) numerical value.

To cancel an error state.

=head3 CLOCK_ADJUST

Reset Minutes & Seconds to closest hour. 

Example: If sent between 13:30:00 and 14:29:00, sets clock to 14:00::00.  
  
=head3 COUNTER_CODE
  
Returns 6 bytes indicating the counter position. 

The first byte will return 0x30 for +, 0x31 for -
Example: "-1:23:45" would return 0x31,0x31,0x32,0x33,0x34,0x35. (Ascii: 112345)

=head3 COUNTER_MEMORY_TOGGLE
  
To toggle Counter Memory On/Off. 

=head3 COUNTER_RESET
  
To reset the counter "00:00:00".  

=head3 EJECT
  
To eject the cassette. When the VCR receives this command, it will respond 
with R_ACK,when the tape has completed ejecting, it will respond with 
CASSETTE_OUT.
    
=head3 FWD
  
To start fast-forward mode.

If sent during playback, will start forward picture search mode.

If sent during still mode, will advance 1 image field.

If sent after the command to set alarm search mode, alarm scan or 
day/time search mode, will start the search or scan in the forward direction.
    
=head3 HEAD_TIME
  
Returns 5 bytes indicating usage time of the video head in hours.

Example:12000 = 12000 hours    

=head3 MENU
  
If sent during stop mode, the main menu is displayed.

If sent while a sub menu is being displayed, the menu screen is switched.  

=head3 MENU_EXIT_AUDIO_ON_SEARCH

If sent when a menu is displayed, the menu is cancelled.

If sent to a time-lapse VCR during playback in 12 or 24 mode, will turn on/off 
the playback audio.

If sent during stop mode, it will toggle search modes in the following order: 
alarm search, alarm scan, day/time search mode.

=head3 MENU_RESET

For each displayed menu, it will rest the menu settings to the default values.    

=head3 ON_SCREEN_ON_TOGGLE  

Toggle on-screen display On/Off

=head3 PAUSE_STILL

If sent during playback, will start still mode.

If sent during recording, will start recording pause mode.

=head3 PLAY  

To start playback. 
If sent during recording, will start recording check mode.

=head3 POWER_TIME

Returns 5 bytes indicating the lifetime in power connected state in hours.  

=head3 POWER_TOGGLE

If the unit can be powered off, toggle the button.

=head3 QM_SECURE_ON

Engage the quick method security lock.
  
=head3 QM_SECURE_OFF      

Cancel the quick method security lock.
    
=head3 REC_DUB_REQUEST

Send this command before sending the RECORD command.

=head3 RECORD

To Start Recording.

If sent during single image recording mode, will record at set field.

Send this command ONLY after sending REC_DUB_REQUEST and receiving R_ACK.

=head3 REV_PLAY

If sent during playback, will start reverse playback.
    
=head3 REW

To start rewind mode.

If sent during playback, will start reverse picture search mode.

If sent during still mode, go back 1 image field.

If sent after the command to set alarm search mode, alarm scan or 
day/time search mode, will start the search or scan in the reverse direction.

=head3 SET_RP_UP

If sent while a menu is being displayed, the setting item selection or setting 
will be changed or the default value will be changed (increased).    

Increase tape speed mode (8/24/40hr)
  
=head3 SET_RP_DOWN  

If sent while a menu is being displayed, the setting item selection or setting 
will be changed or the default value will be changed (decreased).    

Decrease tape speed mode (8/24/40hr)

=head3 SHARPNESS_SHARP

If sent during playback, image becomes sharper
  
=head3 SHARPNESS_SOFT

If sent during playback, image becomes softer

=head3 SHIFT_DOWN

If sent while the clock or timer is being set, the previous setting item 
is selected.

If sent while a submenu is displayed, the main menu is displayed.

If sent while the superimpose display is displayed, the display moves down. 
Will not operate while recording.    
  
=head3 SHIFT_RIGHT

If sent while a menu is displayed, the setting item selection is accepted.

If sent while the main menu is being displayed, the selected submenu appears.

If sent while the superimpose display is displayed, the display moves 
towards the right. Will not operate while recording.    

=head3 STATUS_SENSE

See status_sense method      

=head3 STOP

Enter stop mode.

=head3 TIMER_TOGGLE

If not set, this will set the timer recording ON, if set, will set timer 
recording to STANDBY.
  
=head3 TL_ON 

Start control of the VCR.

=head3 TL_OFF

Stop control of the VCR

=head3 TL_STATUS_SENSE

See tl_status_sense_method  

=head3 TRACKING_CENTER

If sent during Playback, will set the tracking to the neutral position.  

=head3 TRACKING_VLOCK_SHIFT_DOWN  

If sent during playback, will set the tracking one step down.

If sent during still mode, will set the vertical alignment one stop down.

If sent while the menu screen or superimpose display is displayed, the 
operation is the same as SHIFT_DOWN.
  
    
=head3 TRACKING_VLOCK_SHIFT_UP

If sent during playback, will set the tracking one step up.

If sent during still mode, will set the vertical alignment one stop up.

If sent while the menu screen or superimpose display is displayed, the 
operation is the same as SHIFT_RIGHT.
    
=head2 sendbyte

=head2 listcmd
Returns a list of known commands

=head2 debug
If true, displays verbose info via warn.
  
=head2 status_sense

=head2 tl_status_sense    
  
  
=head2 dump_status_sense

=head2 dump_tl_status_sense  
  
    
=head1 Determing VCR Status
  
Status of the VCR is acertained by two seperate "sense" commands. The values returned by the following methods will only be as current as the last "sense" command.

=head2 status_sense

The vcr is queried and if successful, 5 bytes of data are returned. The data consists of various bit flags.

SS_ERROR        
  If set, indicates error, When an incorrect command is received, the CLEAR_ERROR or CLEAR command must be issued before any other commands are excepted.
        
SS_CASSETTE_OUT  
  If set, no cassette loaded in VCR.
  
SS_REC_INHIBIT  
  If set, erasure prevention tab removed. 
    
SS_WARNING        
  If set, something is wrong with the VCR.
  
SS_REPEAT_MODE
  If set, in auto-repeat recording mode.
  
SS_ON_SCREEN_ON
  If set, the superimpose display appears on the screen.
  
SS_COUNTER_MEMORY
  If set, the counter memory of the unit is on.

SS_TIMER_REC_ON
  If set, timer record mode on.

SS_REC_MODE
  If set, recording.

SS_EJECT 
  If set, ejecting the cassette.

SS_RVS_PLAY
  If set, in reverse playback mode.
  
SS_STOP_MODE
  If set, in stop mode.
  
SS_REW_MODE
  If set, rewinding.

SS_FFWD_MODE
  If set, fast-forwarding.

SS_PLAY_MODE
  If set, playing back.
  
SS_STILL_MODE
  If set, in playback still or field advance mode.
  
SS_PAUSE_MODE
  If set, in record pause mode.
  
SS_REVIEW_MODE
  If set, in reverse picture search mode.
  
SS_CUE_MODE
  If set, in forward picture search mode.

=head2 tl_status_sense

The vcr is queried and if successful, 5 bytes of data are returned. The first three bytes indicate the record/playback speed mode, the 2 remaining are bit flags.

=head3 TLSS_AUDIO_ON

If set, time lapse audio playback mode on
  
=head3 TLSS_POWER_ON

If set, the power is on
  
=head3 TLSS_TIMER_REC_ON

If set, timer recording mode or or timer recording standby mode on
  
=head3 TLSS_SERIES_REC_ON

If set, series recording mode on. (Daisy chained units)

=head3 TLSS_ONESHOT_REC_ON

If set, single image recording mode on
  
=head3 TLSS_ALARM_REC_ON

If set, alarm recording mode on.
  
=head3 TLSS_ALARM_SEARCH_SET

If set, alarm search being set.
  
=head3 TLSS_ALARM_SEARCH_MODE

If set, alarm search mode on.
  
=head3 TLSS_ALARM_SCAN_MODE

If set, alarm scan mode on.
  
=head3 TLSS_TD_SEARCH_SET

If set, Day/Time search being set.
  
=head3 TLSS_TD_SEARCH_MODE

If set, Day/Time search mode on.

=head3 TLSS_MENU_MODE

If set, Menu is displayed.

=head1 TODO

Currently all commands but the output commands are implemented. (Alarm search, day search, etc).
This will be fixed shortly.  


          

=head1 BUGS



=head1 SUPPORT

Contact me via email and I'll try and help with any problems you're experiencing.


=head1 AUTHOR

	Lee Pumphret
	CPAN ID: SHOTGUN
	
	perl@leeland.net
	http://leeland.net/myperlmodules.html

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=cut

#################### main pod documentation end ###################



